@@ -15,15 +15,19 @@ from google.protobuf import empty_pb2 as google_dot_protobuf_dot_empty__pb2
 
 from immudb import header_manipulator_client_interceptor
 from immudb.handler import (batchGet, batchSet, changePassword, changePermission, createUser,
-                            currentRoot, databaseCreate, databaseList, databaseUse,
-                            get, listUsers, verifiedGet, verifiedSet, setValue, history,
+                            currentRoot, createDatabase, databaseList, deleteKeys, useDatabase,
+                            get, listUsers, sqldescribe, verifiedGet, verifiedSet, setValue, history,
                             scan, reference, verifiedreference, zadd, verifiedzadd,
-                            zscan, healthcheck, txbyid, verifiedtxbyid, sqlexec, sqlquery,
+                            zscan, healthcheck, health, txbyid, verifiedtxbyid, sqlexec, sqlquery,
                             listtables, execAll)
 from immudb.rootService import *
 from immudb.grpc import schema_pb2_grpc
 import warnings
 import ecdsa
+from immudb.datatypes import DeleteKeysRequest
+from immudb.embedded.store import KVMetadata
+
+import datetime
 
 
 class ImmudbClient:
@@ -33,7 +37,7 @@ class ImmudbClient:
             immudUrl = "localhost:3322"
         self.channel = grpc.insecure_channel(immudUrl)
         self.__stub = schema_pb2_grpc.ImmuServiceStub(self.channel)
-        if rs == None:
+        if rs is None:
             self.__rs = RootService()
         else:
             self.__rs = rs
@@ -41,35 +45,12 @@ class ImmudbClient:
         self.loadKey(publicKeyFile)
 
     def loadKey(self, kfile: str):
-        if kfile == None:
+        if kfile is None:
             self.__vk = None
         else:
             with open(kfile) as f:
                 self.__vk = ecdsa.VerifyingKey.from_pem(f.read())
 
-    def login(self, username, password, database=b"defaultdb"):
-        req = schema_pb2_grpc.schema__pb2.LoginRequest(user=bytes(
-            username, encoding='utf-8'), password=bytes(
-                password, encoding='utf-8'
-        ))
-        try:
-            self.__login_response = schema_pb2_grpc.schema__pb2.LoginResponse = \
-                self.__stub.Login(
-                    req
-                )
-        except ValueError as e:
-            raise Exception(
-                "Attempted to login on termninated client, channel has been shutdown") from e
-
-        self.__stub = self.set_token_header_interceptor(self.__login_response)
-        # Select database, modifying stub function accordingly
-        request = schema_pb2_grpc.schema__pb2.Database(databaseName=database)
-        resp = self.__stub.UseDatabase(request)
-        self.__stub = self.set_token_header_interceptor(resp)
-
-        self.__rs.init("%s/%s".format(self.__url, database), self.__stub)
-        return self.__login_response
-
     def shutdown(self):
         self.channel.close()
         self.channel = None
@@ -98,65 +79,45 @@ class ImmudbClient:
     def stub(self):
         return self.__stub
 
+# from here on same order as in Golang ImmuClient interface (pkg/client/client.go)
+
+    # Not implemented: disconnect
+    # Not implemented: isConnected
+    # Not implemented: waitForHealthCheck
     def healthCheck(self):
         return healthcheck.call(self.__stub, self.__rs)
 
-    def get(self, key: bytes):
-        return get.call(self.__stub, self.__rs, key)
+    # Not implemented: connect
 
-    def getValue(self, key: bytes):
-        ret = get.call(self.__stub, self.__rs, key)
-        if ret == None:
-            return None
-        return ret.value
+    def login(self, username, password, database=b"defaultdb"):
+        req = schema_pb2_grpc.schema__pb2.LoginRequest(user=bytes(
+            username, encoding='utf-8'), password=bytes(
+                password, encoding='utf-8'
+        ))
+        try:
+            self.__login_response = schema_pb2_grpc.schema__pb2.LoginResponse = \
+                self.__stub.Login(
+                    req
+                )
+        except ValueError as e:
+            raise Exception(
+                "Attempted to login on termninated client, channel has been shutdown") from e
 
-    def set(self, key: bytes, value: bytes):
-        return setValue.call(self.__stub, self.__rs, key, value)
+        self.__stub = self.set_token_header_interceptor(self.__login_response)
+        # Select database, modifying stub function accordingly
+        request = schema_pb2_grpc.schema__pb2.Database(databaseName=database)
+        resp = self.__stub.UseDatabase(request)
+        self.__stub = self.set_token_header_interceptor(resp)
 
-    def safeGet(self, key: bytes):
-        warnings.warn("Call to deprecated safeGet. Use verifiedGet instead",
-                      category=DeprecationWarning,
-                      stacklevel=2
-                      )
-        return verifiedGet.call(self.__stub, self.__rs, key, verifying_key=self.__vk)
+        self.__rs.init("{}/{}".format(self.__url, database), self.__stub)
+        return self.__login_response
 
-    def verifiedGet(self, key: bytes):
-        return verifiedGet.call(self.__stub, self.__rs, key, verifying_key=self.__vk)
+    def logout(self):
+        self.__stub.Logout(google_dot_protobuf_dot_empty__pb2.Empty())
+        self.__login_response = None
 
-    def verifiedGetAt(self, key: bytes, atTx: int):
-        return verifiedGet.call(self.__stub, self.__rs, key, atTx, self.__vk)
-
-    def safeSet(self, key: bytes, value: bytes):
-        warnings.warn("Call to deprecated safeSet. Use verifiedSet instead",
-                      category=DeprecationWarning,
-                      stacklevel=2
-                      )
-        return verifiedSet.call(self.__stub, self.__rs, key, value)
-
-    def verifiedSet(self, key: bytes, value: bytes):
-        return verifiedSet.call(self.__stub, self.__rs, key, value, self.__vk)
-
-    def getAllValues(self, keys: list):
-        resp = batchGet.call(self.__stub, self.__rs, keys)
-        return resp
-
-    def getAll(self, keys: list):
-        resp = batchGet.call(self.__stub, self.__rs, keys)
-        return {key: value.value for key, value in resp.items()}
-
-    def setAll(self, kv: dict):
-        return batchSet.call(self.__stub, self.__rs, kv)
-
-    def changePassword(self, user, newPassword, oldPassword):
-        request = schema_pb2_grpc.schema__pb2.ChangePasswordRequest(
-            user=bytes(user, encoding='utf-8'),
-            newPassword=bytes(newPassword, encoding='utf-8'),
-            oldPassword=bytes(oldPassword, encoding='utf-8')
-        )
-        return changePassword.call(self.__stub, self.__rs, request)
-
-    def changePermission(self, action, user, database, permissions):
-        return changePermission.call(self.__stub, self.__rs, action, user, database, permissions)
+    # Not implemented: openSession
+    # Not implemented: closeSession
 
     def createUser(self, user, password, permission, database):
         request = schema_pb2_grpc.schema__pb2.CreateUserRequest(
@@ -170,47 +131,108 @@ class ImmudbClient:
     def listUsers(self):
         return listUsers.call(self.__stub, None)
 
+    def changePassword(self, user, newPassword, oldPassword):
+        request = schema_pb2_grpc.schema__pb2.ChangePasswordRequest(
+            user=bytes(user, encoding='utf-8'),
+            newPassword=bytes(newPassword, encoding='utf-8'),
+            oldPassword=bytes(oldPassword, encoding='utf-8')
+        )
+        return changePassword.call(self.__stub, self.__rs, request)
+
+    def changePermission(self, action, user, database, permissions):
+        return changePermission.call(self.__stub, self.__rs, action, user, database, permissions)
+
+    # Not implemented: updateAuthConfig
+    # Not implemented: updateMTLSConfig
+
+    # Not implemented: with[.*]
+
+    # Not implemented: getServiceClient
+    # Not implemented: getOptions
+    # Not implemented: setupDialOptions
+
     def databaseList(self):
         dbs = databaseList.call(self.__stub, self.__rs, None)
         return [x.databaseName for x in dbs.dblist.databases]
 
-    def databaseUse(self, dbName: bytes):
+    # Not implemented: databaseListV2
+
+    def createDatabase(self, dbName: bytes):
         request = schema_pb2_grpc.schema__pb2.Database(databaseName=dbName)
-        resp = databaseUse.call(self.__stub, self.__rs, request)
+        return createDatabase.call(self.__stub, self.__rs, request)
+
+    # Not implemented: createDatabaseV2
+    # Not implemented: loadDatabase
+    # Not implemented: unloadDatabase
+    # Not implemented: deleteDatabase
+
+    def useDatabase(self, dbName: bytes):
+        request = schema_pb2_grpc.schema__pb2.Database(databaseName=dbName)
+        resp = useDatabase.call(self.__stub, self.__rs, request)
         # modify header token accordingly
         self.__stub = self.set_token_header_interceptor(resp)
         self.__rs.init(dbName, self.__stub)
         return resp
 
-    def databaseCreate(self, dbName: bytes):
-        request = schema_pb2_grpc.schema__pb2.Database(databaseName=dbName)
-        return databaseCreate.call(self.__stub, self.__rs, request)
+    # Not implemented: updateDatabase
+    # Not implemented: updateDatabaseV2
+    # Not implemented: getDatabaseSettings
+    # Not implemented: getDatabaseSettingsV2
+
+    # Not implemented: setActiveUser
+
+    # Not implemented: flushIndex
+
+    def compactIndex(self):
+        self.__stub.CompactIndex(google_dot_protobuf_dot_empty__pb2.Empty())
+
+    def health(self):
+        return health.call(self.__stub, self.__rs)
 
     def currentState(self):
         return currentRoot.call(self.__stub, self.__rs, None)
 
+    def set(self, key: bytes, value: bytes):
+        return setValue.call(self.__stub, self.__rs, key, value)
+
+    def verifiedSet(self, key: bytes, value: bytes):
+        return verifiedSet.call(self.__stub, self.__rs, key, value, self.__vk)
+
+    def expireableSet(self, key: bytes, value: bytes, expiresAt: datetime.datetime):
+        metadata = KVMetadata()
+        metadata.ExpiresAt(expiresAt)
+        return setValue.call(self.__stub, self.__rs, key, value, metadata)
+
+    def get(self, key: bytes):
+        return get.call(self.__stub, self.__rs, key)
+
+    # Not implemented: getSince
+    # Not implemented: getAt
+
+    def verifiedGet(self, key: bytes):
+        return verifiedGet.call(self.__stub, self.__rs, key, verifying_key=self.__vk)
+
+    def verifiedGetSince(self, key: bytes, sinceTx: int):
+        return verifiedGet.call(self.__stub, self.__rs, key, sinceTx, self.__vk)
+
+    def verifiedGetAt(self, key: bytes, atTx: int):
+        return verifiedGet.call(self.__stub, self.__rs, key, atTx, self.__vk)
+
     def history(self, key: bytes, offset: int, limit: int, sortorder: bool):
         return history.call(self.__stub, self.__rs, key, offset, limit, sortorder)
 
-    def logout(self):
-        self.__stub.Logout(google_dot_protobuf_dot_empty__pb2.Empty())
-        self.__login_response = None
-
-    def scan(self, key: bytes, prefix: bytes, desc: bool, limit: int, sinceTx: int = None):
-        return scan.call(self.__stub, self.__rs, key, prefix, desc, limit, sinceTx)
-
-    def setReference(self, referredkey: bytes, newkey:  bytes):
-        return reference.call(self.__stub, self.__rs, referredkey, newkey)
-
-    def verifiedSetReference(self, referredkey: bytes, newkey:  bytes):
-        return verifiedreference.call(self.__stub, self.__rs, referredkey, newkey, verifying_key=self.__vk)
-
     def zAdd(self, zset: bytes, score: float, key: bytes, atTx: int = 0):
         return zadd.call(self.__stub, self.__rs, zset, score, key, atTx)
 
     def verifiedZAdd(self, zset: bytes, score: float, key: bytes, atTx: int = 0):
         return verifiedzadd.call(self.__stub, self.__rs, zset, score, key, atTx, self.__vk)
 
+    # Not implemented: zAddAt
+    # Not implemented: verifiedZAddAt
+
+    def scan(self, key: bytes, prefix: bytes, desc: bool, limit: int, sinceTx: int = None):
+        return scan.call(self.__stub, self.__rs, key, prefix, desc, limit, sinceTx)
+
     def zScan(self, zset: bytes, seekKey: bytes, seekScore: float,
               seekAtTx: int, inclusive: bool, limit: int, desc: bool, minscore: float,
               maxscore: float, sinceTx=None, nowait=False):
@@ -224,6 +246,42 @@ class ImmudbClient:
     def verifiedTxById(self, tx: int):
         return verifiedtxbyid.call(self.__stub, self.__rs, tx, self.__vk)
 
+    # Not implemented: txByIDWithSpec
+
+    # Not implemented: txScan
+
+    # Not implemented: count
+    # Not implemented: countAll
+
+    def setAll(self, kv: dict):
+        return batchSet.call(self.__stub, self.__rs, kv)
+
+    def getAll(self, keys: list):
+        resp = batchGet.call(self.__stub, self.__rs, keys)
+        return {key: value.value for key, value in resp.items()}
+
+    def delete(self, req: DeleteKeysRequest):
+        return deleteKeys.call(self.__stub, req)
+
+    def execAll(self, ops: list, noWait=False):
+        return execAll.call(self.__stub, self.__rs, ops, noWait)
+
+    def setReference(self, referredkey: bytes, newkey:  bytes):
+        return reference.call(self.__stub, self.__rs, referredkey, newkey)
+
+    def verifiedSetReference(self, referredkey: bytes, newkey:  bytes):
+        return verifiedreference.call(self.__stub, self.__rs, referredkey, newkey, verifying_key=self.__vk)
+
+    # Not implemented: setReferenceAt
+    # Not implemented: verifiedSetReferenceAt
+
+    # Not implemented: dump
+
+    # Not implemented: stream[.*]
+
+    # Not implemented: exportTx
+    # Not implemented: replicateTx
+
     def sqlExec(self, stmt, params={}, noWait=False):
         """Executes an SQL statement
         Args:
@@ -264,8 +322,51 @@ class ImmudbClient:
         """
         return listtables.call(self.__stub, self.__rs)
 
-    def execAll(self, ops: list, noWait=False):
-        return execAll.call(self.__stub, self.__rs, ops, noWait)
+    def describeTable(self, table):
+        return sqldescribe.call(self.__stub, self.__rs, table)
 
-    def compactIndex(self):
-        self.__stub.CompactIndex(google_dot_protobuf_dot_empty__pb2.Empty())
+    # Not implemented: verifyRow
+    # Not implemented: newTx
+
+# deprecated
+    def databaseCreate(self, dbName: bytes):
+        warnings.warn("Call to deprecated databaseCreate. Use createDatabase instead",
+                      category=DeprecationWarning,
+                      stacklevel=2
+                      )
+        return self.createDatabase(dbName)
+
+    def safeGet(self, key: bytes):  # deprecated
+        warnings.warn("Call to deprecated safeGet. Use verifiedGet instead",
+                      category=DeprecationWarning,
+                      stacklevel=2
+                      )
+        return verifiedGet.call(self.__stub, self.__rs, key, verifying_key=self.__vk)
+
+    def databaseUse(self, dbName: bytes):  # deprecated
+        warnings.warn("Call to deprecated databaseUse. Use useDatabase instead",
+                      category=DeprecationWarning,
+                      stacklevel=2
+                      )
+        return self.useDatabase(dbName)
+
+    def safeSet(self, key: bytes, value: bytes):  # deprecated
+        warnings.warn("Call to deprecated safeSet. Use verifiedSet instead",
+                      category=DeprecationWarning,
+                      stacklevel=2
+                      )
+        return verifiedSet.call(self.__stub, self.__rs, key, value)
+
+
+# immudb-py only
+
+
+    def getAllValues(self, keys: list):  # immudb-py only
+        resp = batchGet.call(self.__stub, self.__rs, keys)
+        return resp
+
+    def getValue(self, key: bytes):  # immudb-py only
+        ret = get.call(self.__stub, self.__rs, key)
+        if ret is None:
+            return None
+        return ret.value
